@@ -1,12 +1,18 @@
-GeoNA=function(data,coordx,coordy=NULL, coordt=NULL, coordx_dyn=NULL, grid=FALSE, X=NULL,setting="spatial")
+GeoNA=function(data,coordx,coordy=NULL,coordz=NULL, coordt=NULL, 
+      coordx_dyn=NULL, grid=FALSE, X=NULL,setting="spatial")
 {
 
 ## handling spatial coordinates
    if(is.null(coordy)) coords=as.matrix(coordx)
     else{
-    if(grid) coords=as.matrix(expand.grid(coordx,coordy))
-    else     coords=cbind(coordx,coordy)  
-    }
+        if(grid) 
+            {if(is.null(coordz)) {coords=as.matrix(expand.grid(coordx,coordy)) }
+             else                {coords=as.matrix(expand.grid(coordx,coordy,coordz))}
+            }
+        else {if(is.null(coordz)) {coords=cbind(coordx,coordy)  }
+              else   {coords=cbind(coordx,coordy,coordz)  }
+            }
+  }
 ####    
 N=length(c(unlist(data)))  #length of data
 
@@ -30,6 +36,7 @@ if (setting=='bivariate'){
 }
 nonan = list( coordx = coords,
               coordy = coordy,
+              coordz = coordz,
               coordt = coordt,
               coordx_dyn=coordx_dyn,              
               data=data,
