@@ -4,7 +4,7 @@
 
 ### Procedures are in alphabetical order.
 
-GeoVariogram <- function(data, coordx, coordy=NULL, coordt=NULL, coordx_dyn=NULL,cloud=FALSE, distance="Eucl",
+GeoVariogram <- function(data, coordx, coordy=NULL, coordz=NULL,coordt=NULL, coordx_dyn=NULL,cloud=FALSE, distance="Eucl",
                        grid=FALSE, maxdist=NULL,neighb=NULL, maxtime=NULL,numbins=NULL,
                        radius=6371, type='variogram',bivariate=FALSE)
   {
@@ -25,9 +25,10 @@ GeoVariogram <- function(data, coordx, coordy=NULL, coordt=NULL, coordx_dyn=NULL
       if(is.numeric(coordt))
         if(length(coordt)>1) corrmodel <- 'gneiting'
     # Checks the input:
-    checkinput <- CkInput(coordx, coordy, coordt, coordx_dyn, corrmodel, data, distance, "Fitting", NULL, grid,
+
+    checkinput <- CkInput(coordx, coordy, coordz,coordt, coordx_dyn, corrmodel, data, distance, "Fitting", NULL, grid,
                              'None', maxdist, maxtime, model,NULL, 'Nelder-Mead', NULL,
-                             radius,  NULL, NULL,NULL, 'GeoWLS', FALSE, 'SubSamp', FALSE,NULL,NULL)
+                             radius,  NULL, NULL,NULL, 'GeoWLS', FALSE, FALSE,NULL,NULL)
                              
 
     # Checks if there are errors in the input:
@@ -47,14 +48,18 @@ GeoVariogram <- function(data, coordx, coordy=NULL, coordt=NULL, coordx_dyn=NULL
   
 
     n=1
-    initparam <- StartParam(coordx, coordy, coordt,coordx_dyn, corrmodel, data,distance, "Fitting",
-                           NULL, grid, 'None', maxdist,neighb,
-                           maxtime, model, n, NULL, NULL, FALSE, radius, 
-                           NULL, NULL, NULL, 'GeoWLS', 'GeoWLS', FALSE,
-                           'SubSamp', FALSE, 1, 1,1,1,NULL,NULL,FALSE)
+
+    initparam <- StartParam(coordx, coordy, coordz,coordt,coordx_dyn, corrmodel, data,distance, "Fitting",
+                           NULL, grid, 'None', maxdist,neighb, maxtime, model, n, 
+                           NULL, NULL, FALSE, radius, NULL, NULL, NULL,
+                            'GeoWLS', 'GeoWLS', FALSE,NULL,NULL,FALSE,FALSE)
+
+
 
     spacetime_dyn=NULL
-    coordx=initparam$coordx;coordy=initparam$coordy;coordt=initparam$coordt                 
+    coordx=initparam$coordx;coordy=initparam$coordy;coordz=initparam$coordz;
+    coordt=initparam$coordt  
+              
     # Checks if there are inconsistences:
     if(!is.null(initparam$error))
       stop(initparam$error)
@@ -87,9 +92,14 @@ GeoVariogram <- function(data, coordx, coordy=NULL, coordt=NULL, coordx_dyn=NULL
     NS=cumsum(ns)
     if(!spacetime_dyn){  data=c(t(data))
                          if(memdist){coordx=rep(coordx,n_var)
-                                     coordy=rep(coordy,n_var)}
+                                     coordy=rep(coordy,n_var)
+                                     if(!is.null(coordz))  coordz=rep(coordz,n_var)
+                                   }
                       }
-    if(spacetime_dyn) data=unlist(data)
+    if(spacetime_dyn) {data=unlist(data)}
+    
+
+    if(is.null(coordz)) coordz=double(length(coordx))
     NS=c(0,NS)[-(length(ns)+1)]
     if(!memdist)  {  
                moments_marg<-double(n_var*numvario)   # vect of square differences for each component (n_var) 11  e 22
@@ -97,9 +107,9 @@ GeoVariogram <- function(data, coordx, coordy=NULL, coordt=NULL, coordx_dyn=NULL
                moments_cross<-double(0.5*n_var*(n_var-1)*numvario)  # vect of square differences for cross components (12)
                lenbins_cross<-integer(0.5*n_var*(n_var-1)*numvario) #
                DEV=dotCall64::.C64("Binned_Variogram_biv2", 
-                     SIGNATURE = c("double","double","double","double","double", "integer","double","integer","integer","double","integer","integer"),
-                      bins=bins, coordx,coordy,coordt,data,lenbins_cross=lenbins_cross, moments_cross=moments_cross, numbins,lenbins_marg=lenbins_marg,
-                       moments_marg=moments_marg,ns,NS,INTENT = c("rw","r","r","r","r","rw","rw","rw","rw","rw","r","r"),NAOK = TRUE, PACKAGE = "GeoModels", VERBOSE = 0)
+                     SIGNATURE = c("double","double","double","double","double","double", "integer","double","integer","integer","double","integer","integer"),
+                      bins=bins, coordx,coordy,coordz,coordt,data,lenbins_cross=lenbins_cross, moments_cross=moments_cross, numbins,lenbins_marg=lenbins_marg,
+                       moments_marg=moments_marg,ns,NS,INTENT = c("rw","r","r","r","r","r","rw","rw","rw","rw","rw","r","r"),NAOK = TRUE, PACKAGE = "GeoModels", VERBOSE = 0)
                bins=DEV$bins
                lenbins_cross=DEV$lenbins_cross; moments_cross=DEV$moments_cross
                lenbins_marg=DEV$lenbins_marg; moments_marg=DEV$moments_marg
@@ -117,11 +127,12 @@ GeoVariogram <- function(data, coordx, coordy=NULL, coordt=NULL, coordx_dyn=NULL
                variogramst=variograms_12
             }   
              else {
-             idx=GeoNeighIndex(coordx=cbind(coordx,coordy),coordx_dyn=coordx_dyn,
+             idx=GeoNeighIndex(coordx=cbind(coordx,coordy,coordz),coordx_dyn=coordx_dyn,
                   distance = distance, neighb = neighb, maxdist = maxdist,maxtime=1,radius=1,bivariate=TRUE)
              mm=range(idx$lags)
              moments00=double(numvario);moments10=double(numvario);moments11=double(numvario)
              lenbins00=double(numvario);lenbins10=double(numvario);lenbins11=double(numvario);
+
 DEV=dotCall64::.C64("Binned_Variogram_biv2new", 
                     SIGNATURE = c("double","integer","double","double", 
                       "double","double","double","double","double",
@@ -165,14 +176,16 @@ if(initparam$spacetime){
       #if(cloud) fname <- 'Cloud_Variogram_st' else 
       fname <- 'Binned_Variogram_st'
 
-      if(grid)     {a=expand.grid(coordx,coordy);coordx=a[,1];coordy=a[,2]; }
+      if(grid)     {
+          if(is.null(coordz)) {a=expand.grid(coordx,coordy);coordx=a[,1];coordy=a[,2]}
+          else                {a=expand.grid(coordx,coordy,coordz);coordx=a[,1];coordy=a[,2];coordz=a[,3]}
+           }
       else{
-              if(!spacetime_dyn){ data=c(t(data))
-                          coordx=rep(coordx,numtime)
-                          coordy=rep(coordy,numtime)
-                         }
-               if(spacetime_dyn) data=unlist(data)
+              if(!spacetime_dyn) data=c(t(data))
+              else               data=unlist(data)
       }
+   
+
        NS=c(0,NS)[-(length(ns)+1)]
       if(spacetime_dyn) fname <- paste(fname,"2_dyn",sep="") 
       if(!spacetime_dyn) fname <- paste(fname,"2",sep="") 
@@ -189,25 +202,23 @@ if(initparam$spacetime){
            bint[1]=0;
            for(h in 2:(numbint)) bint[h]=bint[h-1]+step
          }
-#################################
-  
+#########################
+ if(is.null(coordz))  coordz=double(length(coordx)*(length(coordx)-1))
   if(!memdist)  { 
                     EV=dotCall64::.C64(fname, 
-                    SIGNATURE = c("double","double","double","double","double","double",
+                    SIGNATURE = c("double","double","double","double","double","double","double",
                                "integer","integer","integer","double","double","double",
                                 "integer","integer","integer","integer"),
-                                 bins=bins, bint=bint,coordx,coordy,coordt,data,lenbins=lenbins,lenbinst=lenbinst,lenbint=lenbint,
+                                 bins=bins, bint=bint,coordx,coordy,coordz,coordt,data,lenbins=lenbins,lenbinst=lenbinst,lenbint=lenbint,
                                  moments=moments,momentst=momentst,momentt=momentt,numbins,numbint,ns,NS,
-                                 INTENT = c("rw","rw","r","r","r","r","rw","rw","rw","rw","rw","rw", "r","r","r","r"), 
+                                 INTENT = c("rw","rw","r","r","r","r","r","rw","rw","rw","rw","rw","rw", "r","r","r","r"), 
                                  NAOK = TRUE, PACKAGE = "GeoModels", VERBOSE = 0)
     }
     else {
              stop("not implemented")
-             idx=GeoNeighIndex(cbind(coordx,coordy),coordt=coordt,coordx_dyn=coordx_dyn,distance = distance, neighb = neighb, maxdist = maxdist,maxtime=maxtime,radius=radius)
+             idx=GeoNeighIndex(cbind(coordx,coordy,coordz),coordt=coordt,coordx_dyn=coordx_dyn,distance = distance, neighb = neighb, maxdist = maxdist,maxtime=maxtime,radius=radius)
         }
-     #print(bint)
-     #print(numbint)
-     #print(regular)
+ 
 
        bins=EV$bins;   lenbins=EV$lenbins
        bint=EV$bint;  lenbint=EV$lenbint
@@ -237,31 +248,40 @@ if(initparam$spacetime){
       memdist=FALSE
   if(!is.null(neighb)) memdist=TRUE
      
-    if(grid)     {a=expand.grid(coordx,coordy);coordx=a[,1];coordy=a[,2]; }
+    if(grid)     {
+                      if(is.null(coordz)) {a=expand.grid(coordx,coordy);coordx=a[,1];coordy=a[,2];}
+                      else {a=expand.grid(coordx,coordy,coordz);coordx=a[,1];coordy=a[,2];coordz=a[,3];}
+
+                  }
+
+ if(is.null(coordz)) coordz=double(length(coordx))
 
      if(!memdist)  { 
      fname <- paste(fname,"2",sep="") 
      # Computes the spatial moments
      # EV=.C("Binned_Variogram2", bins=bins,  as.double(coordx),as.double(coordy),as.double(coordt),as.double(data), 
       #  lenbins=lenbins, moments=moments, as.integer(numbins),PACKAGE='GeoModels', DUP = TRUE, NAOK=TRUE)
+
+  
+
         EV=dotCall64::.C64("Binned_Variogram2",
-         SIGNATURE = c("double","double","double","double","double","integer","double","integer"),
-            bins=bins, coordx,coordy,coordt,data, lenbins=lenbins, moments=moments, numbins,
-                INTENT = c("w","r","r","r","r","w","w","r"), 
+         SIGNATURE = c("double","double","double","double","double","double","integer","double","integer"),
+            bins=bins, coordx,coordy,coordz,coordt,data, lenbins=lenbins, moments=moments, numbins,
+                INTENT = c("w","r","r","r","r","r","w","w","r"), 
                      NAOK = TRUE, PACKAGE = "GeoModels", VERBOSE = 0)
 
        }
       else {
        fname="Binned_Variogram2new"
   
-         idx=GeoNeighIndex(cbind(coordx,coordy),distance = distance, neighb = neighb, maxdist = maxdist,radius=radius)
+         idx=GeoNeighIndex(cbind(coordx,coordy,coordz),distance = distance, neighb = neighb, maxdist = maxdist,radius=radius)
          #mm=c(min(idx$lags),max(idx$lags))
          mm=range(idx$lags)
         
         # EV=.C("Binned_Variogram2new", bins=bins,  as.integer(length(idx$lags)),as.double(data[idx$colidx]),
         #        as.double(data[idx$rowidx]), as.double(idx$lags),
         #lenbins=lenbins, moments=moments, as.integer(numbins),as.double(mm),PACKAGE='GeoModels', DUP = TRUE, NAOK=TRUE)
-   
+
    EV=dotCall64::.C64("Binned_Variogram2new", 
                     SIGNATURE = c("double","integer","double","double", 
                       "double","integer","double","integer","double"),
